﻿enyo.kind({
    name: "enyo.CacheSource",
    kind: enyo.Component,
    components: [
        { name: "downloader",
            kind: "PalmService",
            service: "palm://com.palm.downloadmanager/",
            method: "download",
            onResponse: "gotResponse",
            subscribe: true
        },
    ],

    file: null, // reference on file plugin

    // used GlobalMapper format
    // can be exported from SAS.Planeta

    cacheDirectory: "/media/internal/yandexmaps/cache/",
    cacheYandex: ".yandex/",
    cacheGoogle: ".google/",
    cacheOSM: ".osm/",

    setPlugin: function() {
        this.file = enyo.application.filePlugin;
        console.log("file cache is READY");
    },

    inDownload: 0,

    get: function (url) {

        var layerPrefix = this.cacheYandex;
        if (url.indexOf('google') > 0)
            layerPrefix = this.cacheGoogle;
        else if (url.indexOf('openstreetmap') > 0)
            layerPrefix = this.cacheOSM;

        var z = 0;
        var y = 0;
        var x = 0;

        var ofilename = null;
        var filename = null;
        switch (layerPrefix) {
            case this.cacheYandex:
                filename = url.split('?')[1].replace(/&/g, '') + ".png";
                ofilename = filename;

                var zpatt = /z=\d+lang=/;
                var ypatt = /y=\d+z=/;
                var xpatt = /x=\d+y=/;
    
                if (zpatt.test(filename)) {
                    var zres = zpatt.exec(filename);
                    z = /\d+/.exec(zres);
                }

                if (ypatt.test(filename)) {
                    var yres = ypatt.exec(filename);
                    y = /\d+/.exec(yres);
                }

                if (xpatt.test(filename)) {
                    var xres = xpatt.exec(filename);
                    x = /\d+/.exec(xres);
                }
                layerPrefix += "z"+ z +"/"+ y +"/";
                filename = ""+ x + ".png";
                break;
            case this.cacheOSM:
                filename = url.substr(url.indexOf('openstreetmap.org/')+18).replace(/&/g, '').replace(/\//g, '_');
                ofilename = filename;

                var sar = filename.split('_');

                layerPrefix += "z"+ sar[0] +"/"+ sar[1] +"/";
                filename = sar[2];

                break;
            default:
                filename = url.substr(url.lastIndexOf('/')).replace(/&/g, '').replace(/\//g, '_') + ".png";
                ofilename = filename;

                var zpatt = /z=\d+\./;
                var ypatt = /y=\d+z=/;
                var xpatt = /x=\d+y=/;
    
                if (zpatt.test(filename)) {
                    var zres = zpatt.exec(filename);
                    z = /\d+/.exec(zres);
                }

                if (ypatt.test(filename)) {
                    var yres = ypatt.exec(filename);
                    y = /\d+/.exec(yres);
                }

                if (xpatt.test(filename)) {
                    var xres = xpatt.exec(filename);
                    x = /\d+/.exec(xres);
                }
                layerPrefix += "z"+ z +"/"+ y +"/";
                filename = ""+ x + ".png";

                break;
        }

        // do not cache traffic tiles
        if (ofilename.indexOf('l=trf') >= 0)
            return url;

        var fullname = this.cacheDirectory + layerPrefix + filename;
        //console.log("file: " + fullname);
        if (this.file) {
            var fileExist = this.file.exist(fullname);
            //console.log("exist: " + fileExist);
            if (!fileExist) {
                //console.log("downloading file: " + filename);
                this.$.downloader.call({
                    target: url,
                    mime: "image/png",
                    targetDir: this.cacheDirectory + layerPrefix,
                    targetFilename: filename,
                    keepFilenameOnRedirect: false,
                    canHandlePause: false,
                    subscribe: true
                });

                this.inDownload++;

                console.log('downloading: ' + this.inDownload);
            } else {
                // return from file cache
                //console.log("get from cache: " + fullname);
                return fullname;
            }
        }
        else {
            //console.log("file plugin is not ready");
        }
        return url;
    },

    retryList: {},
    gotResponse: function (inSender, inResponse) {
        //console.log('down: ' + enyo.json.stringify(inResponse));
        if (inResponse.returnValue != 'undefined') {
            if (inResponse.completed || inResponse.interrupted || inResponse.aborted) {
                if (inResponse.completionStatusCode != 200) {
                    console.log('file cache failed: [' + inResponse.url + '], code: '+ inResponse.completionStatusCode);
                    
                    if (inResponse.url in this.retryList) {
                        this.retryList[inResponse.url].count--;
                        if (this.retryList[inResponse.url].count > 0) {

                            console.log('retry [' +this.retryList[inResponse.url].count +'] download: '+ inResponse.url);

                            this.$.downloader.call({
                                target: inResponse.url,
                                mime: "image/png",
                                targetDir: inResponse.destPath,
                                targetFilename: inResponse.destFile,
                                keepFilenameOnRedirect: false,
                                canHandlePause: false,
                                subscribe: true
                            });
                            this.inDownload++;
                        }
                        else {
                            delete this.retryList[inResponse.url];
                        }
                    }
                    else {
                        this.retryList[inResponse.url] = { count: 5 };

                        console.log('retry [' +this.retryList[inResponse.url].count +'] download: '+ inResponse.url);

                        this.$.downloader.call({
                            target: inResponse.url,
                            mime: "image/png",
                            targetDir: inResponse.destPath,
                            targetFilename: inResponse.destFile,
                            keepFilenameOnRedirect: false,
                            canHandlePause: false,
                            subscribe: true
                        });

                        this.inDownload++;
                    } 
                }

                this.inDownload--;
                console.log('downloading: ' + this.inDownload);
            }
        }
    },
})